# -*- coding: UTF-8 -*-

import decimal
import json
import os
from common import filesmodify, logger, macros
from datetime import datetime, date

logInfo = "AIF converter: "
saleFiles = set()


def JSONAdditions(o):
    if isinstance(o, decimal.Decimal):
        return float(o)
    if isinstance(o, datetime):
        return '{0.year:4d}-{0.month:02d}-{0.day:02d}T{0.hour:02d}:{0.minute:02d}:{0.second:02d}.{0.microsecond:06d}'.format(o)
    if isinstance(o, date):
        return '{0.year:4d}-{0.month:02d}-{0.day:02d}T00:00:00.000000'.format(o)
    raise TypeError(repr(o) + " is not JSON serializable")


def parseFlag(flag):
    f = open(flag, 'r')
    shifts = None
    dates = None
    if f.readline().strip() == '###':
        str = f.readline().strip()
        if len(str.split('-')) == 2:
            if str.find('.') == -1:
                shifts = str.replace('-', '@')
            else:
                fromDate = str.split('-')[0].split('.')
                toDate = str.split('-')[1].split('.')
                dates = '%s-%s-%sT00:00:00.0@%s-%s-%sT23:59:59.99' % (fromDate[2], fromDate[1], fromDate[0], toDate[2], toDate[1], toDate[0])
    f.close()
    return shifts, dates


def writeDictObjectAsJson(f, dictObject):
    f.write(json.dumps(dictObject, sort_keys=True, ensure_ascii=True, indent=2, default=JSONAdditions).encode('UTF-8'))
    f.write('\n---\n')


def run(salesFunc, securityLogsFunc, shiftsFunc, cash, node, workDirectory, currentSales):
    log = logger.getUnloadLogger("Node", node.id)
    log.info(logInfo + "Start unload sales for cash №" + str(cash.code))
    resultPath = node.currentConverter.onlineSalePath if currentSales else node.currentConverter.path
    log.info('currentSales=%s resultPath=%s'%(currentSales,resultPath))
    log.info('shopcode=%s cashcode=%s'%(node.id,cash.code))
    currentPath = macros.applyMacros(resultPath, log, shopcode = node.id, cashcode = cash.code)
    log.info('currentPath=%s'%(currentPath))
    salePath = workDirectory + '/' + os.path.basename(currentPath)
    log.info('workDirectory=%s'%(workDirectory))
#    salePath = workDirectory + '/' + os.path.basename(node.currentConverter.path)
    if os.path.isfile(salePath):
        log.info("%sfile %s exists. Delete old sales" % (logInfo, salePath))
    else:
        log.info("%sCreate file %s" % (logInfo, salePath))
    f = open(salePath, 'w')
    emptyFile = True
    f.write('### Artix:Interchange datafile, version 1.1\n')
    log.info('node.currentConverter.dataType = %s' % node.currentConverter.dataType)
    if node.currentConverter.dataType in ['sales','all']:
        f.write('### sales data begin ###\n')
        cur = f.tell()
        for i in salesFunc():
            writeDictObjectAsJson(f, i)
        if cur != f.tell():
            emptyFile = False
        f.write('### sales data end ###\n')

    if node.currentConverter.dataType in ['crights','all']:
        f.write('### securitylog info begin ###\n')
        cur = f.tell()
        for i in securityLogsFunc():
            writeDictObjectAsJson(f, i)
        if cur != f.tell():
            emptyFile = False
        f.write('### securitylog info end ###\n')

    if node.currentConverter.dataType in ['sales','all']:
        f.write('### shift info begin ###\n')
        cur = f.tell()
        for i in shiftsFunc():
            if not emptyFile or (node.currentConverter.withClosedShifts and i['timeEnd']):
                writeDictObjectAsJson(f, i)
        if node.currentConverter.withClosedShifts and cur != f.tell():
            emptyFile = False
        f.write('### shift info end ###\n')
    f.close()

    log.info("%ssales generated" % logInfo)
    if emptyFile:
        log.info('Sales data is empty. Delete empty file.')
        os.remove(salePath)
    else:
        statusFileName = "ready"
        statusDir = workDirectory
        if currentSales:
            statusFileName += "Online"
            statusDir = os.path.dirname(workDirectory)
        converterSalesReadyFile = open('%s/%s' % (statusDir, statusFileName), 'w')
        converterSalesReadyFile.close()
        salesReadyFile = open('%s/%s' % (os.path.dirname(os.path.dirname(statusDir)), statusFileName), 'w')
        salesReadyFile.close()
        global saleFiles
        saleFiles = set((salePath, converterSalesReadyFile.name, salesReadyFile.name))


def sendSales(cash, node, workDirectory, backupDir):
    log = logger.getUnloadLogger("Node", node.id)
    return filesmodify.safeMoveDirectory(log, workDirectory, os.path.dirname(node.currentConverter.path),
            rewrite=node.currentConverter.rewriteDestFile, backupDir=backupDir,
            maxCountFiles=cash.maxCountFiles, actualCountDays=cash.actualCountDays)


def sendOnlineSales(cash, node, workDirectory):
    log = logger.getUnloadLogger("Node", node.id)
    return filesmodify.safeMoveDirectory(log, workDirectory, os.path.dirname(node.currentConverter.onlineSalePath))


def getSaleFiles():
    global saleFiles
    return saleFiles
