# -*- coding: UTF-8 -*-

import os
from common import filesmodify, logger, macros
import codecs

logInfo = "NS2000 converter: "
currentConverter = None
saleFiles = set()


def openFile(log, cash, node, workDirectory, shiftNum, previousFile=None, isClFile=True):
    output = None
    firstRound = True
    headerSize = 8
    salePathMask = os.path.join(currentConverter.path, '%(cashcode_int)02d%(shiftnum_int)06d.cl')

    if not isClFile:
        headerSize = 5
        salePathMask = os.path.join(currentConverter.path, '%(cashcode_int)02d%(shiftnum_int)06d.sk')
    currentPath = macros.applyMacros(salePathMask, log, shopcode=node.id, cashcode=cash.code, shiftnum=shiftNum)
    salePath = os.path.join(workDirectory, os.path.basename(currentPath))
    if os.path.isfile(salePath):
        log.info(logInfo + "file exists " + salePath)
        f = codecs.open(salePath, 'r', currentConverter.charset)
        if f.readline(1):
            if not currentConverter.rewriteDestFile or previousFile == salePath:
                output = codecs.open(salePath, 'a', currentConverter.charset)
        try:
            for i in range(headerSize):
                f.next()
                firstRound = False
        except StopIteration:
            firstRound = True
            if output:
                output.close()
        f.close()
        firstRound = False
    if not output or output.closed:
        output = codecs.open(salePath, 'w', currentConverter.charset)
        firstRound = True
        log.info(logInfo + "file is created " + salePath)
    global saleFiles
    saleFiles.add(output.name)
    return output, firstRound


def run(salesFunc, securityLogsFunc, shiftsFunc, cash, node, workDirectory, currentSales):
    global currentConverter
    log = logger.getUnloadLogger("Node", node.id)
    log.info(logInfo + "START. Cash №" + str(cash.code))
    currentConverter = node.currentConverter
    output = None
    outputSk = None
    firstRound = True       # Флаг. Указавет на то, чтобы писать или не писать заголовок отчета в файл.
    shifts = {}
    userNames = {}
    for sh in shiftsFunc():
        shifts[sh['shift']] = sh
        userNames[sh['shift']] = {}
        for user in sh['users']:
            userNames[sh['shift']][user['usercode']] = user['username']
    shift = None
    for check in salesFunc():
        if firstRound:
            output, firstRound = openFile(log, cash, node, workDirectory, str(check['shift']))
            outputSk, firstRoundSk = openFile(log, cash, node, workDirectory, str(check['shift']), isClFile=False)
        if shift != check['shift']:
            if shift in shifts:
                previousFile = output.name
                previousFileSk = outputSk.name
                output.close()
                output, firstRound = openFile(log, cash, node, workDirectory, str(check['shift']), previousFile)
                outputSk, firstRoundSk = openFile(log, cash, node, workDirectory, str(check['shift']), previousFileSk, isClFile=False)
            shift = check['shift']

        discSum = {}
        cardNumber = ''
        action = 'A'
        shiftEnd = shifts[check['shift']]['timeEnd']
        if firstRound:
            output.write(u"FOR NS2000 ЛЕНТА ВЫГРУЖЕНА " + (shiftEnd.strftime('%d.%m.%Y %H:%M') if shiftEnd else '') + "\n")
            output.write(u"КАССА=," + str(check['cashCode']).zfill(2) + "\n")
            output.write(u"СМЕНА=," + str(check['shift']).zfill(6) + "\n")
            output.write(u"ДАТАНАЧ=," + (shiftEnd.strftime('%d/%m/%Y') if shiftEnd else '') + "\n")
            output.write(u"ДАТАКОН=," + (shiftEnd.strftime('%d/%m/%Y') if shiftEnd else '') + "\n")
            output.write(u"ВРЕМЯ=," +  (shiftEnd.strftime('%H:%M:%S') if shiftEnd else '') + "\n")
            output.write(u"КАССИР=," + str(check['userCode']) + "\n")
            output.write(u"ТИП=ALL\n")
            firstRound = False
        if firstRoundSk:
            outputSk.write(u"ДАТА=" +  (shiftEnd.strftime('%d/%m/%Y') if shiftEnd else '') + "\n")
            outputSk.write(u"ВРЕМЯ=" +  (shiftEnd.strftime('%H:%M') if shiftEnd else '') + "\n")
            outputSk.write(u"КАССА=" + str(check['cashCode']).zfill(2) + "\n")
            outputSk.write(u"СЕАНС=" + str(check['cashCode']).zfill(2) + '/' + str(check['shift']).zfill(6) + "\n")
            if check['userCode'] in userNames[check['shift']]:
                user = userNames[check['shift']][check['userCode']]
            else:
                user = ''
            outputSk.write(u"КАССИР=" + user + "\n")
            firstRoundSk = False

        for moneyPos in check['moneyPositions']:
            strTrans = 'CM=,%s,%s,%s,%s,%s\n' % (check['docNum'],
                                                            str(check['docType']).zfill(2),
                                                            moneyPos['valCode'],
                                                            ('-' if moneyPos['opCode'] in [72,74,82] else '+') + str(moneyPos['sumB']),
                                                            ('-' if moneyPos['opCode'] in [72,74,82] else '+') + str(moneyPos['sumB']))
            output.write(strTrans)
        strTrans = 'C=,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s\n' % (check['docNum'],
                                                                               check['timeBeg'].strftime('%d/%m/%Y'),
                                                                               check['timeBeg'].strftime('%H:%M'),
                                                                               str(check['docType']).zfill(2),
                                                                               check['sumtype'],
                                                                               str(check['docSum']).rjust(12),
                                                                               str(check['sumn']).rjust(12),
                                                                               str(check['disc_abs']).rjust(12),
                                                                               str(check['disc_perc']).rjust(12),
                                                                               check['userCode'])
        output.write(strTrans)
        for invent in check['inventPositions']:
            strTrans = 'CT=,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s\n' % (
                                                                              check['docNum'],
                                                                              invent['posTime'].strftime('%d/%m/%Y'),
                                                                              invent['posTime'].strftime('%H:%M'),
                                                                              str(check['docType']).zfill(2),
                                                                              invent['deptCode'],
                                                                              invent['barCode'],
                                                                              invent['articul'].rjust(10),
                                                                              invent['name'],
                                                                              str(invent['price']).rjust(12),
                                                                              ("%.3f" % invent['quant']).zfill(9),
                                                                              ("%.2f" % invent['posSum']).zfill(11),
                                                                              ("%.2f" % (invent['quant'] * invent['price'])).zfill(11),
                                                                              ("%.2f" % invent['finalPrice']).zfill(11),
                                                                              "%SINFO%",
                                                                              userNames[check['shift']][check['userCode']])
            output.write(strTrans)
            for discount in invent['discountPositions']:
                if (discount['discMode'] == 'byCard'):
                    cardNumber = discount['cardNumber']
                    type = discount['discType']
                    action = 'p' if type == 'percent' else 'a' if type == 'summ' else 'i' if type == 'index' else 'b' if type == 'bonus' else 'a'
                    if action not in discSum:
                        discSum[action] = 0
                    discSum[action] += discount['discSum']
        if discSum:
            for key, value in discSum.items():
                strDisc = 'C=%s,%s,%s,%s,%s,%s,%s%s%s \n' % (
                                                                              str(check['cashCode']).zfill(2),
                                                                              str(check['shift']).zfill(6),
                                                                              str(check['docNum']).zfill(8),
                                                                              cardNumber,
                                                                              ("%.2f" % check['baseSum']).rjust(12),
                                                                              ("%.2f" % check['docSum']).rjust(12),
                                                                              ("%.2f" % value ).rjust(12),
                                                                              " % ",
                                                                              key)
                outputSk.write(strDisc)

    if output:
        output.close()
        converterSalesReadyFile = open('%s/ready' % workDirectory, 'w')
        converterSalesReadyFile.close()
        salesReadyFile = open('%s/ready' % (os.path.dirname(os.path.dirname(workDirectory))), 'w')
        salesReadyFile.close()
        saleFiles.add(converterSalesReadyFile.name)
        saleFiles.add(salesReadyFile.name)

    if outputSk:
        outputSk.close()

    if shift:
        shifts[shift] = ''
    log.info(logInfo + "Unloaded the following number of shifts: %s" % shifts.keys())
    log.info(logInfo + "FINISH")


def sendSales(cash, node, workDirectory, backupDir):
    log = logger.getUnloadLogger("Node", node.id)
    return filesmodify.safeMoveDirectory(log, workDirectory, node.currentConverter.path, backupDir=backupDir,
            rewrite=node.currentConverter.rewriteDestFile, maxCountFiles=cash.maxCountFiles, actualCountDays=cash.actualCountDays)


def sendOnlineSales(cash, node, workDirectory):
    return False, []


def getSaleFiles():
    global saleFiles
    return saleFiles
