# -*- coding: UTF-8 -*-

import codecs
import os
from common import filesmodify, logger, macros
import datetime

logInfo = "Pilot-original converter: "
currentConverter = None
saleFiles = set()


def openFile(log, cash, node, workDirectory, shiftNum, previousFile=None):
    global saleFiles
    output = None
    firstRound = True
    headerSize = 8

    currentPath = macros.applyMacros(currentConverter.path, log, shopcode=node.id, cashcode=cash.code, shiftnum=shiftNum)
    salePath = os.path.join(workDirectory, os.path.basename(currentPath))
    if os.path.isfile(salePath):
        log.info(logInfo + "file exists " + salePath)
        f = codecs.open(salePath, 'r', currentConverter.charset)
        if f.readline(1):
            if not currentConverter.rewriteDestFile or previousFile == salePath:
                output = codecs.open(salePath, 'a', currentConverter.charset)
        try:
            for i in range(headerSize):
                f.next()
                firstRound = False
        except StopIteration:
            firstRound = True
            if output:
                output.close()
        f.close()
        firstRound = False
    if not output or output.closed:
        output = codecs.open(salePath, 'w', currentConverter.charset)
        firstRound = True
        log.info(logInfo + "file is created " + salePath)
    saleFiles.add(output.name)
    return output, firstRound


def run(salesFunc, securityLogsFunc, shiftsFunc, cash, node, workDirectory, currentSales):
    global currentConverter
    global saleFiles
    log = logger.getUnloadLogger("Node", node.id)
    log.info(logInfo + "START. Cash №" + str(cash.code))
    currentConverter = node.currentConverter
    output = None
    firstRound = True       # Флаг. Указавет на то, чтобы писать или не писать заголовок отчета в файл.
    shifts = {}
    userNames = {}
    for sh in shiftsFunc():
        shifts[sh['shift']] = sh
        userNames[sh['shift']] = {}
        for user in sh['users']:
            userNames[sh['shift']][user['usercode']] = user['username']
    shift = None
    for check in salesFunc():
        if check['closed'] != 1:
            continue

        if check['docType'] == 1:
            checkType = 1
        elif check['docType'] in [2,25]:
            checkType = 2
        else:
            continue

        if firstRound:
            output, firstRound = openFile(log, cash, node, workDirectory, str(check['shift']))
        if shift != check['shift']:
            if shifts.has_key(shift):
                previousFile = output.name
                output.close()
                output, firstRound = openFile(log, cash, node, workDirectory, str(check['shift']), previousFile)
            shift = check['shift']

        shiftBeg = shifts[check['shift']]['timeBeg']
        if firstRound:
                output.write(u"КОНТРОЛЬНАЯ ЛЕНТА\r\n")
                output.write(u"ДАТА=" + (shiftBeg.strftime('%d/%m/%Y') if shiftBeg else '') + "\r\n")
                output.write(u"ВРЕМЯ=" + (shiftBeg.strftime('%H:%M:%S') if shiftBeg else '') + "\r\n")
                output.write(u"КАССА=" + str(check['cashCode']).zfill(3) + "\r\n")
                output.write(u"СЕАНС=" + str(check['cashCode']).zfill(3) + "/" + str(check['shift']).zfill(6) + "\r\n")
                output.write(u"КАССИР=" + userNames[check['shift']][check['userCode']] + "\r\n")
                output.write(u"ТИП=ИТОГОВАЯ\r\n\r\n")
                firstRound = False

        strTrans = 'C=%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s\r\n' % (str(check['docNum']).rjust(8),
                                                    check['timeBeg'].strftime('%d/%m/%Y'),
                                                    check['timeBeg'].strftime('%H:%M:%S'),
                                                    str(checkType).zfill(2),
                                                    '   1',
                                                    str(check['docSum']).rjust(13),
                                                    str(check['docSum']).rjust(13),
                                                    str(check['disc_perc']).rjust(13),
                                                    str(check['disc_abs']).rjust(13),
                                                    str(check['userCode']).rjust(4),
                                                    '',
                                                    '')
        output.write(strTrans)

        for moneyPos in check['moneyPositions']:
            strTrans = 'CM=%s,%s,%s,%s,%s,%s,%s,%s\r\n' % (str(check['docNum']).rjust(7),
                                                    moneyPos['posTime'].strftime('%d/%m/%Y'),
                                                    moneyPos['posTime'].strftime('%H:%M:%S'),
                                                    str(checkType).zfill(2),
                                                    '   4' if moneyPos['valutOperation'] else '   1',
                                                    ('-' if moneyPos['opCode'] in [72, 76] else '+') + ("%.4f" % moneyPos['sumB']).zfill(12),
                                                    ('-' if moneyPos['opCode'] in [72, 76] else '+') + ("%.4f" % moneyPos['sumB']).zfill(12),
                                                    ('-' if moneyPos['opCode'] in [72, 76] else '+') + ("%.4f" % moneyPos['sumB']).zfill(12))
            output.write(strTrans)

        for invent in check['inventPositions']:
            strTrans = 'CT=%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s\r\n' % (
                                                    str(check['docNum']).rjust(7),
                                                    invent['posTime'].strftime('%d/%m/%Y'),
                                                    invent['posTime'].strftime('%H:%M:%S'),
                                                    str(checkType).zfill(2),
                                                    str(invent['deptCode'] if invent['deptCode'] > 0 else 1).rjust(4),
                                                    str(invent['barCode']).ljust(20),
                                                    str(invent['price']).rjust(13),
                                                    ("%.2f" % invent['posSum']).rjust(13),
                                                    ("%.2f" % invent['posSum']).rjust(13),
                                                    ('-' if checkType == 2 else '+') + ("%.3f" % invent['quant']).zfill(9),
                                                    str(invent['disc_perc']).rjust(6),
                                                    str(invent['disc_abs']).rjust(12),
                                                    str(check['userCode']).rjust(4),
                                                    invent['articul'].ljust(10),
                                                    '',
                                                    '',
                                                    '   1')
            output.write(strTrans)

    if output:
        output.close()
        converterSalesReadyFile = open('%s/ready' % (workDirectory), 'w')
        converterSalesReadyFile.close()
        salesReadyFile = open('%s/ready' % (os.path.dirname(os.path.dirname(workDirectory))), 'w')
        salesReadyFile.close()
        saleFiles.add(converterSalesReadyFile.name)
        saleFiles.add(salesReadyFile.name)

    if shift:
        shifts[shift] = ''
    log.info(logInfo + "Unloaded the following number of shifts: %s" % shifts.keys())
    log.info(logInfo + "FINISH")


def sendSales(cash, node, workDirectory, backupDir):
    log = logger.getUnloadLogger("Node", node.id)
    return filesmodify.safeMoveDirectory(log, workDirectory, os.path.dirname(node.currentConverter.path),
            backupDir=backupDir, rewrite=node.currentConverter.rewriteDestFile, maxCountFiles=cash.maxCountFiles, actualCountDays=cash.actualCountDays)


def sendOnlineSales(cash, node, workDirectory):
    return False, []


def getSaleFiles():
    global saleFiles
    return saleFiles
