# -*- coding: UTF-8 -*-
'''
Created on 05.02.2013

@author: chvv
'''

import datetime
from common import aif_objects as dicts

class AtolFormatter(object):
    def __init__(self, delimiter=';'):
        self.useMultiplicityQuantityForMeasure = False
        self.delimiter = delimiter
        self.aspect = {}
        self.emptyCommands = {
            '$$$DELETEALLWARES': [{"command": "clearInventory"},
                                  {"command": "clearInventGroup"}],             # удалить все товары и группы товаров
            '$$$DELETEALLBARCODES': {"command": "clearBarcode"},                # удалить все штрихкоды
            # '$$$DELETEALLCURRENCIES': {"command": "clearCurrency"},          # удалить валюты
            '$$$DELETEALLPAYMENTS': {"command": "clearCurrency"},               # удалить все виды оплат
            '$$$DELETEALLUSERS': {"command": "clearMCashUser"},                 # удалить всех пользователей
            '$$$DELETEALLAUTODISCSCHMS': {"command": "clearLDiscScheme"},       # удалить все схемы внутренних автоматических скидок
            '$$$DELETEALLAUTODISCCONDS': {"command": "clearDiscountsAuto"},     # удалить все внутренние автоматические скидки
            '$$$DELETEALLFIXDISCS': {"command": "clearLDiscFixed"},             # удалить все фиксированные скидки
            '$$$DELETEALLCCARDDISCS': {"command": "clearCard"},                 # удалить все карты и объекты дисконтирования с картами
            '$$$DELETEALLCLIENTDISCS': {"command": "clearClient"},              # удалить всех клиентов
            '$$$DELETEALLPRICES': {"command": "clearPrice"},                    # удалить все цены
            '$$$DELETEALLCLASSIFIERS': {"command": "clearCatalog"},             # удалить все классификаторы
            '$$$DELETECLASSIFIERLINKS': {"command": "clearInventCatalog"},      # удалить связь товара, клиента, карты, предприятия с классификаторами
            '$$$DELETEALLWARESCALELP': {"command": "clearTmcScale"},            # удаление всех дополнительных команд для ВПЭ
            # '$$$DELETEALLDISCOBJECTS':                                       # удалить все объекты скидок
            # '$$$DELETEALLDISCRATES':                                         # удалить все ставки и группы ставок
            '$$$DELETEALLTAXRATES':  {"command": "clearLVat"},                  # удалить все налоговые ставки
            # '$$$DELETEALLTAXGROUPS':                                         # удалить все налоговые группы
            # '$$$DELETEALLTAXGROUPRATES':                                     # удалить все налоговые ставки группы
            '$$$DELETEALLASPECTVALUES': {"command": "clearAspectValueSet"},     # удалить все значения разрезов
            '$$$DELETEALLASPECTSCHMS': {"command": "clearAspectScheme"},        # удалить все схемы разрезов
        }
        self.commands = {
            '$$$ADDQUANTITY': lambda line: self.addInventItem(line),                # добавить товар с добавлением начального остатка (ПО УМОЛЧАНИЮ ИСПОЛЬЗУЕТСЯ ДАННАЯ КОМАНДА)
            '$$$REPLACEQUANTITY': lambda line: self.addInventItem(line),            # добавить товар с замещением начального остатка и удалением продаж;
            '$$$REPLACEQUANTITYWITHOUTSALE': lambda line: self.addInventItem(line), # добавить товар с замещением начального остатка
            '$$$DELETEWARESBYWARECODE': lambda line: self.deleteInventItem(line),   # удалить товары по коду
            # '$$$DELETEBARCODESBYWARECODE':                                       # удалить штрихкоды товара по коду товара
            '$$$ADDBARCODES': lambda line: self.addBarcode(line),                   # добавить штрихкоды
            # '$$$ADDCURRENCIES': lambda line: self.addCurrency(line),             # добавить валюты
            '$$$ADDPAYMENTS': lambda line: self.addCurrency(line),                  # добавить виды оплат
            # '$$$ADDDISCOBJECTS':                                                 # добавить объекты скидок
            # '$$$ADDDISCRATES':                                                   # добавить ставки и группы ставок
            '$$$ADDFIXDISCS': lambda line: self.addLDiscFixed(line),                # добавить фиксированные скидки
            # '$$$ADDAUTODISCSCHMS': lambda line: self.addLDiscScheme(line),       # добавить схемы внутренних автоматических скидок
            '$$$ADDAUTODISCCONDS': lambda line: self.addDiscountsAuto(line),        # добавить внутренние автоматические скидки
            # '$$$DELETEDISCSCHMSBYCODE':                                          # удалить условия и ссылающиеся на них объекты скидок
            # '$$$DELETEDISCCONDSBYCODE':                                          # удалить элементы условий
            '$$$ADDTAXRATES': lambda line: self.addTax(line),                       # добавить налоговые ставки
            # '$$$ADDTAXGROUPS':                                                   # добавить налоговые группы
            '$$$ADDTAXGROUPRATES': lambda line: self.addTaxGroupRates(line),        # добавить налоговые ставки группы
            '$$$ADDCCARDDISCS': lambda line: self.addDiscCard(line),                # добавить карты
            '$$$ADDCLIENTDISCS': lambda line: self.addClient(line),                 # добавить клиентов
            # '$$$ADDCLIENTCCARDS':                                                # добавить связь карты с клиентом
            # '$$$DELETECLIENTCCARDS':                                             # удалить связь клиента и карты
            # '$$$DELETEALLCLIENTCCARDS':                                          # удалить все привязки карт к клиентам
            '$$$ADDUSERS': lambda line: self.addMCashUser(line),                    # добавить пользователей
            '$$$DELETEUSERSBYCODE': lambda line: self.delMCashUser(line),           # удалить пользователя по коду
            # '$$$ADDEMPLOYEES': ,                                                 # добавить сотрудников
            # '$$$DELETEALLEMPLOYEES': ,                                           # удалить всех сотрудников
            '$$$ADDPRICES': lambda line: self.addPrice(line),                       # добавить цены
            '$$$DELETEPRICESBYWARE': lambda line: self.delPrice(line),              # удалить цены товаров
            '$$$ADDCLIENTCCARDS': lambda line: self.setCardClient(line),            # добавить связь карты с клиентом
            '$$$ADDCLASSIFIERS': lambda line: self.addCatalog(line),                # добавить классификаторы
            '$$$ADDCLASSIFIERLINKS': lambda line: self.addInventCatalog(line),      # добавить связь товара, клиента, карты, предприятия с классификаторами
            '$$$ADDWARESCALELP': lambda line: self.addTmcScale(line),               # добавить связь товара, клиента, карты, предприятия с классификаторами
            '$$$DELETEWARESCALELP': lambda line: self.deleteTmcScale(line),         # добавить связь товара, клиента, карты, предприятия с классификаторами
            '$$$ADDASPECTSCHMS': lambda line: self.addAspectScheme(line),           # добавить схемы разрезов
            '$$$ADDASPECTVALUES': lambda line: self.addAspectValueSet(line),        # добавить значения разрезов
            '$$$ADDASPECTREMAINS': lambda line: self.addAspectRemains(line),        # добавить остатки по разрезам с добавлением начального остатка;
            '$$$REPLACEASPECTREMAINSWITHOUTSALE': lambda line: self.addAspectRemains(line),  # добавить остатки по разрезам с замещением начального остатка.
        }
            #$$$REPLACEASPECTREMAINS             # Добавить остатки по разрезам с замещением начального остатка и удалением продаж;
            #$$$DELETEALLASPECTREMAINS           # Удалить все остатки по разрезам
            #$$$ADDWAREANALOGS                   # Добавить аналоги товаров
            #$$$DELETEALLWAREANALOGS             # Удалить все аналоги товаров
            #$$$DELETEASPECTREMAINSBYWARECODE    # Удалить остатки по разрезам товара
            #$$$ADDASPECTS                       # Добавить разрезы
            #$$$DELETEALLASPECTS                 # Удалить все разрезы
            #$$$ADDDENOMINATIONS                 # Добавить купюры
            #$$$DELETEALLDENOMINATIONS           # Удалить все купюры
            #$$$ADDCARDGROUPS                    # Добавить группу сертификатов
            #$$$DELETEALLCARDGROUPS              # Удалить все группы сертификатов
            #$$$ADDCARDS                         # Добавить сертификаты
            #$$$DELETEALLCARDS                   # Удалить все сертификаты
            #$$$ADDCOMMENTS                      # Добавить комментарии
            #$$$DELETEALLCOMMENTS                # Удалить все комментарии
            #$$$DELETEALLCLASSIFIERLINKS         # Удалить все связи товара, клиента, карты с классификаторами
            #$$$ADDSETTINGS                      # Загрузить настройки
        self.minprices = {}

    def getValuesFromString(self, string):
        array = string.strip().split(';')
        for i in range(len(array)):
            if array[i] == '':
                array[i] = None
        for i in range(len(array), 57):
            array.append(None)
        return array

    def toDate(self, string):
        if string:
            return str(datetime.datetime.strptime(string, '%d.%m.%Y' if len(string) > 8 else '%d.%m.%y').date())
        return None

    def toDateTime(self, dateStr, timeStr):
        if dateStr and timeStr:
            timeMask = '%H:%M' if len(timeStr) == 5 else '%H.%M.%S'
            dateMask = '%d.%m.%Y' if len(dateStr) > 8 else '%d.%m.%y'
            return str(datetime.datetime.strptime(dateStr + "_" + timeStr, dateMask + '_' + timeMask))
        return None

    def getFloatParam(self, value, defaultValue=0.0):
        return float(value.replace(',', '.')) if value else defaultValue

    def getIntParam(self, arg, defaultValue=0):
        try:
            return int(arg)
        except Exception:
            return defaultValue

    def addInventItem(self, line):
        values = self.getValuesFromString(line)
        if values[16] != '1':
            return self.addInventGroup(line)

        measurecode = '2'  # весовой
        if values[7][0] == '0':
            measurecode = '1'  # штучный
        if self.useMultiplicityQuantityForMeasure and values[32] and self.getFloatParam(values[32]) < 1.0:
            measurecode = '3'  # дробный
        price = self.getFloatParam(values[4])
        remain = self.getFloatParam(values[5])

        minDisc = price * (100 - self.getFloatParam(values[28])) / 100 if values[28] else 0.0
        minPrice = max(minDisc, self.getFloatParam(values[8]))

        options = None
        taraCapacity = None
        alcTypeCode = None
        alcoholPercent = None
        tmctype = None

        if values[54] == '1':
            alcTypeCode, taraCapacity, options, alcoholPercent = self.getEgaisProperties(values)
        elif values[54] == '3':
            tmctype = dicts.Barcode.MEDICINE
        elif values[54] in {'4', '12', '101'}:
            options = {"inventitemoptions": {"tobacco": 1}}
            tmctype = dicts.Barcode.TOBACCO
        elif values[54] == '5':
            options = {"inventitemoptions": {"shoes": 1}}
            tmctype = dicts.Barcode.SHOES
        elif values[54] in {'2', '7', '8', '9', '10', '11', '13'}:
            tmctype = dicts.Barcode.MARKEDGOODS

        result = []
        barcodes = []
        if values[1]:
            for bcode in values[1].split(','):
                barcodes.append({"barcode": bcode.strip(), "tmctype": tmctype})

        result.append(dicts.Invent().addCmd(inventcode=values[0], barcode=None, name=values[2], price=price,
                remain=remain, aspectschemecode=values[10], minprice=minPrice,
                taxgroupcode=self.getIntParam(values[22]), articul=values[25], measurecode=measurecode,
                deptcode=values[36], inventgroup=values[15], options=options, taracapacity=taraCapacity,
                alctypecode=alcTypeCode, alcoholpercent=alcoholPercent, barcodes=barcodes))  # cquant=values[32])
        self.minprices[values[0]] = minPrice
        return result

    def addInventGroup(self, line):
        values = self.getValuesFromString(line)
        return dicts.InventGroup().addCmd(groupcode=values[0], parentgroupcode=values[15], groupname=values[2])

    def deleteInventItem(self, line):
        values = self.getValuesFromString(line)
        return dicts.Invent().deleteCmd(values[0])

    def addBarcode(self, line):
        values = self.getValuesFromString(line)
        return dicts.Barcode().addCmd(code=values[1], barcode=values[0], aspectvaluesetcode=values[2])

    def addCurrency(self, line):
        values = self.getValuesFromString(line)
        mode = {'denyMakeChange': (values[3] == '0')}  # нельзя давать сдачу
        return dicts.Currency().addCmd(currencycode=values[0], shortname=values[1],
                                       fullname=values[2], type=values[4], mode=mode)

    def addMCashUser(self, line):
        values = self.getValuesFromString(line)
        roleUsers = [{"rolecode": values[3], "rule": "1"}]
        return dicts.MCashUser().addCmd(code=values[0], login=values[0], name=values[2],
                                        password=values[4], roleusers=roleUsers)

    def delMCashUser(self, line):
        values = self.getValuesFromString(line)
        return dicts.MCashUser().deleteCmd(code=values[0])

    def addLDiscScheme(self, line):
        values = self.getValuesFromString(line)
        isCardScheme = 1 if values[40] == '1' else 0
        return dicts.LDiscScheme().addCmd(schemecode=values[0], name=values[0], iscardscheme=isCardScheme)

    def addDiscountsAuto(self, line):
        result = [self.addLDiscScheme(line)]
        values = self.getValuesFromString(line)
        datebeg = self.toDate(values[6])
        dateend = self.toDate(values[7])
        discType = '2'  # надбавка
        if values[4] in ['0', '1']:
            discType = '1'  # скидка
        discMode = '2'  # суммовая
        if values[4] in ['0', '2']:
            discMode = '1'  # процентная
        sumbeg = values[18]
        sumend = values[19]
        rsummode = '0'
        if values[21] and values[22]:
            rsummode = '1'
            sumbeg = values[21]
            sumend = values[22]
        result.append(dicts.DiscountsAuto().addCmd(schemecode=values[0], disccode=values[1], name=values[2],
                disctype=discType, discmode=discMode, discsize=values[5], rtext=values[3], datebeg=datebeg,
                dateend=dateend, timebeg=values[9], timeend=values[10], daysbeg=self.getIntParam(values[12]) + 1,
                daysend=self.getIntParam(values[13]) + 1, quantbeg=values[15], quantend=values[16], sumbeg=sumbeg,
                sumend=sumend, rsummode=rsummode, catalogid=self.getIntParam(values[33], None),
                catalogquantbeg=values[34], catalogquantend=values[35], catalogsumbeg=values[37],
                catalogsumend=values[38],)) # cardgroupcode=values[19], birthdaymode=values[20], pricecode=values[17]
        return result

    def addLDiscFixed(self, line):
        values = self.getValuesFromString(line)
        discType = '2'  # надбавка
        if values[4] in ['0', '1']:
            discType = '1'  # скидка
        discMode = '2'  # суммовая
        if values[4] in ['0', '2']:
            discMode = '1'  # процентная
        receiptmode = values[8]
        positionmode = 0 if values[8] == '1' else 1
        return dicts.LDiscFixed().addCmd(disccode=values[0], name=values[1], disctype=discType,
                                   discmode=discMode, discsize=values[5], rtext=values[2],
                                   cardpref=values[3], receiptmode=receiptmode, positionmode=positionmode)

    def addDiscCard(self, line):
        values = self.getValuesFromString(line)
        blocked = 0 if values[5] == '1' else 1
        return dicts.Card().addCmd(idcard=values[0], idcardgroup=None, number=values[2], blocked=blocked,
                validitydatebeg=self.toDate(values[3]), validitydateend=self.toDate(values[4]), cardsum=values[10])

    def addClient(self, line):
        values = self.getValuesFromString(line)
        if values[4] == '0':  # если клиент
            sex = None
            if values[6] == '1':
                sex = 1
            elif values[6] == '2':
                sex = 0
            return dicts.Client().addCmd(idclient=values[0], name=values[2], text=values[3], sex=sex,
                    birthday=values[8], specialdate1=values[7], zipcode=values[10], address=values[12],
                    email=values[13], webpage=values[14], phonenumber=values[15], inn=values[16],
                    document=values[17], okpo=values[18], okpd=values[19], extendedoptions=values[5])

    def addPrice(self, line):
        values = self.getValuesFromString(line)
        minPrice = self.minprices[values[0]] if values[0] in self.minprices else 0.0
        return dicts.Price().addCmd(barcode=values[0], price=values[3],
                documentid=self.toDateTime(values[4], values[5]), effectivedate=self.toDateTime(values[4], values[5]),
                doctype=1, minprice=minPrice)

    def delPrice(self, line):
        values = self.getValuesFromString(line)
        return dicts.Price().deleteCmd(barcode=values[0], documentid=self.toDateTime(values[4], values[5]), doctype=1)

    def setCardClient(self, line):
        values = self.getValuesFromString(line)
        return dicts.Card().updateCmd(idcard=values[2], idclient=values[0])

    def addCatalog(self, line):
        values = self.getValuesFromString(line)
        # не группы классификаторов
        if values[2] == '0':
            return dicts.Catalog().addCmd(catalogid=values[0], catalogname=values[3])

    def addInventCatalog(self, line):
        values = self.getValuesFromString(line)
        # только для товаров
        if values[1] == '1':
            return dicts.InventCatalog().addCmd(tmccode=values[2], catalogid=values[0])

    def getEgaisProperties(self, values):
        '''
        Значения по умолчанию из документации(frontol v5.9):
            Код вида алкогольной продукции: 10
            Ёмкость тары: 1.0
            Маркировка алкогольной продукции: с акцизной маркой
            Крепость алкогольной продукции: 0.1
        '''
        alcTypeCode = self.getIntParam(values[52], 10)
        taraCapacity = self.getFloatParam(values[53], 1.0)
        options = {'inventitemoptions': {}}
        if values[55] and values[55] == '1':
            options['inventitemoptions']['nopdfegaisverify'] = 1
        else:
            options['inventitemoptions']['egaisverify'] = 1
        alcoholPercent = self.getFloatParam(values[56], 0.1)
        return alcTypeCode, taraCapacity, options, alcoholPercent

    def addTmcScale(self, line):
        values = self.getValuesFromString(line)
        return dicts.TmcScale().addCmd(tmccode=values[0], tmcscalegroupcode=values[1], plu=values[2],
                tmcscalecode=values[3], ingredients=None, manufacturer=None, prefix=values[6])

    def deleteTmcScale(self, line):
        values = self.getValuesFromString(line)
        return dicts.TmcScale().deleteCmd(tmccode=values[0],
                                          tmcscalecode=None)  # Код ВПЭ при выгрузке не используется, нужно чтоб коды товаров в выгрузке были уникальны!

    def addAspectScheme(self, line):
        values = self.getValuesFromString(line)
        return dicts.AspectScheme().addCmd(aspectschemecode=values[0], aspectschemename=values[1])

    def addAspectValueSet(self, line):
        values = self.getValuesFromString(line)
        self.aspect[values[2]] = values[0]
        return dicts.AspectValueSet().addCmd(aspectschemecode=values[0], aspectvaluesetcode=values[2], name=values[3])

    def addAspectRemains(self, line):
        values = self.getValuesFromString(line)
        aspectremains = {
            'aspectschemecode': self.aspect[values[2]],
            'aspectvaluesetcode': values[2],
            'price': values[3],
            'remain': self.getFloatParam(values[1])
        }
        return dicts.AddAspectRemains(inventcode=values[0], aspectremains=aspectremains).dict

    def addTax(self, line):
        values = self.getValuesFromString(line)
        taxes = [{"name": values[1], "rate": values[4]}]
        return dicts.TaxGroup().addCmd(idtaxgroup=self.getIntParam(values[0]), taxes=taxes)

    def addTaxGroupRates(self, line):
        values = self.getValuesFromString(line)
        return dicts.UpdateTaxCode(taxcode=self.getIntParam(values[2]), taxgroupcode=self.getIntParam(values[1]),
                                   mode=values[3]).dict
