# -*- coding: UTF-8 -*-

from common import aif_objects as aif, logger
from upload.inserter import InserterError

import xml.etree.ElementTree as ET

errorObjectsCount = 0

class Converter(object):
    def __init__(self, workDirectory, data, node):
        self.workDirectory = workDirectory
        self.data = data
        self.node = node
        self.log = logger.getUploadLogger("Node", node.id)
        plugin = __import__('upload.inserter.%s' % (node.inserter.name), fromlist='Inserter')
        cl = getattr(plugin, 'Inserter')
        self.inserter = cl(node)
        self.listObjects = []
        # переоценки
        self.prices = []
        # дополнительные штрих-коды
        self.additionalBarcodes = []
        #additional prices
        self.additionalPrices = []
        # группы товаров
        self.groups = {}
        # min prices
        self.minprices = {}
        # max discounts
        self.maxdiscounts = {}

    def getMinPrice(self, inventcode, price):
        minprice = 0
        if str(inventcode) in self.minprices:
            mp = float(self.minprices[str(inventcode)])
            if minprice < mp:
                minprice = mp
        if str(inventcode) in self.maxdiscounts:
            md = float(self.maxdiscounts[str(inventcode)])
            if minprice < price - md*price / 100:
                minprice = price - md*price / 100
        return round(minprice, 2)


    def appendDict(self, aif_object):
        self.listObjects.append(aif_object)
        if len(self.listObjects) == self.node.pack_size:
            self.inserter.add_package(self.listObjects)
            self.listObjects = []

    def addPrice(self, prc, barcode, minprice):
        self.prices.append(aif.Price().addCmd(barcode = barcode, price = prc.get('price'), documentid = prc.find('begin-date').text, 
            effectivedate = prc.find('begin-date').text, doctype = 1, minprice = minprice))
    
    def getMinPrices(self, root):
        for restriction in root.findall('min-price-restriction'):
            if restriction.find('deleted').text == 'false':
                self.minprices[str(restriction.get('subject-code'))] = restriction.get('value')
                
    def getMaxDiscounts(self, root):
        for restriction in root.findall('max-discount-restriction'):
            if restriction.find('deleted').text == 'false':
                self.maxdiscounts[str(restriction.get('subject-code'))] = restriction.get('value')
        

    def getPrinciplePrice(self, good):
        for prc in good.findall('price-entry'):
            if prc.find('number').text == '1':
                price = prc.get('price')
                deptcode = prc.find('department').get('number')
                inventcode = good.get('marking-of-the-good')
                minprice = self.getMinPrice(inventcode, float(price))

                self.addPrice(prc, inventcode, minprice)
                break
        return (price, deptcode, minprice)

    def parseGroup(self, group):
        if group.find('parent-group') != None:
            self.groups[group.get('id')] = ["" if group.find('name') is None else group.find('name').text, 
                                            "" if group.find('parent-group') is None else group.find('parent-group').get('id')]
            self.parseGroup(group.find('parent-group'))
        else: 
            self.groups[group.get('id')] = ["" if group.find('name') is None else group.find('name').text, None]

    def parseBarcode(self, brc, inventcode, price, name, measurecode, rootMarkType):
        barcode = brc.get('code')
        quantdefault = brc.find('count').text
        tmp_price = price

        for prc in brc.findall('price-entry'):
            if prc.find('number').text == '1':
                tmp_price = prc.get('price')
                self.addPrice(prc, barcode, self.getMinPrice(inventcode, float(tmp_price)))

        tmcType = None
        if brc.get("marked") and brc.get("marked") == "true" and rootMarkType == "MARKEDGOODS":
            tmcType = aif.Barcode.MARKEDGOODS 

        self.additionalBarcodes.append(aif.Barcode().addCmd(code = inventcode, barcode = barcode, price=0, 
            name=name, quantdefault=quantdefault, tmctype = tmcType))

    def parseAdditionalPrices(self, mrc, inventcode):
        hasprices = 0
        for pr in mrc.findall('plugin-property'):
            if pr.get('key') == 'price':
                hasprices += 1
                self.additionalPrices.append(aif.AdditionalPrice().addCmd(barcode=inventcode, 
                                                            pricecode=None, price=pr.get('value')))
        return hasprices

    def parseSaleRestrictProduct(self, good, options):
        if not 'inventitemoptions' in options:
            options['inventitemoptions'] = {}
        alcoholpercent = None
        taracapacity = None
        
        # Получить признак алкогольной продукции
        exciseField = good.find('excise')
        if exciseField != None and exciseField.text.lower() == 'true':
            options['inventitemoptions']['egaisverify'] = 1

        if good.find('product-type').text == 'ProductSpiritsEntity':
            options['inventitemoptions']['requiresalerestrict'] = 1
            for plugin in good.findall('plugin-property'):
                # Получить крепость алкогольной продукции
                if plugin.get('key') == 'alcoholic-content-percentage':
                    alcoholpercent = plugin.get('value')
                # Получить объём алкогольной продукции(taracapacity)
                elif plugin.get('key') == 'volume':
                    taracapacity = plugin.get('value')
        elif good.find('product-type').text == 'ProductCiggyEntity':
            options['inventitemoptions']['requiresalerestrict'] = 1
            for plugin in good.findall('plugin-property'):
                if plugin.get('key') == 'mrc':
                    hasprices = self.parseAdditionalPrices(plugin, good.get('marking-of-the-good'))
                    if hasprices:
                        #options['priceoptions'] = {}
                        options['priceoptions']['requireselectprice'] = 1
        
        return (alcoholpercent, taracapacity)

    def parseSaleGroup(self, good, options):
        salegroupid = str(good.find('sale-group').get('id'))
        if salegroupid in self.node.currentConverter.visualVerifySGs:
            if not 'inventitemoptions' in options:
                options['inventitemoptions'] = {}
            options['inventitemoptions']['visualverify'] = 1
        if salegroupid in self.node.currentConverter.requireQuantityManualSGs:
            if not 'quantityoptions' in options:
                options['quantityoptions'] = {}
            options['quantityoptions']['requirequantitymanual'] = 1

    def simpleCond(self, good):
        if good.get('marking-of-the-good')!=None and good.find('name')!=None and good.find('bar-code')!=None:
            return True
        else:
            return False
        
    def deleteGoodFromCash(self, obj):
        if obj.find('delete-from-cash') != None:
            if obj.find('delete-from-cash').text == 'true':
                return True;
        return False;

    def deleteObjectFromCash(self, obj):
        if obj.get('deleted') != None:
            if obj.get('deleted') == 'true':
                return True;
        return False;

    def parseGood(self, good):
        inventcode = good.get('marking-of-the-good')
        if not self.deleteGoodFromCash(good):
            name = good.find('name').text
    
            if good.find('measure-type').get('id') in self.node.currentConverter.measureMap:
                measurecode = self.node.currentConverter.measureMap[good.find('measure-type').get('id')]
            else:
                measurecode = good.find('measure-type').get('id')
            options = {}
            
            options['priceoptions'] = {}
            options['priceoptions']['requiredeferredprice'] = 1
          
            alcoholpercent, taracapacity = self.parseSaleRestrictProduct(good, options)

            if good.find('sale-group') != None:
                self.parseSaleGroup(good, options)
            
            inventgroup = None if good.find('group') is None else good.find('group').get('id')
            if good.find('group') != None:
                self.parseGroup(good.find('group'))
            
            price, deptcode, minprice = self.getPrinciplePrice(good)

            type = good.find('mark-type')
            if type != None and type.text == 'FOOTWEAR':
                markType = 'shoes'
            elif type == None or type.text == 'TOBACCO':
                markType = 'tobacco'
            elif type != None:
                # базовая маркировка
                markType = "MARKEDGOODS"

            # если в корне объекта есть признак табака или обуви и есть штрих-код с полем marked = true
            # устанавливаем тип маркировки в опции товара
            for brc in good.findall('bar-code'):
                if brc.get('marked') == 'true' and markType != "MARKEDGOODS":
                    options['inventitemoptions'][markType] = 1
                    break
            
            # если не табак и не обувь, то проставляем базовую маркировку у штрих-кодов с полем marked = true
            for brc in good.findall('bar-code'):
                self.parseBarcode(brc, inventcode, price, name, measurecode, markType)

            return aif.Invent().addCmd(inventcode = inventcode, barcode = None, name = name, deptcode = deptcode, measurecode = measurecode, 
                                 price = 0, inventgroup = inventgroup, alcoholpercent = alcoholpercent, options = options, minprice = 0, 
                                 taracapacity = taracapacity)
        else:
            return aif.Invent().deleteCmd(inventcode=inventcode)

    def parseClientAndCards(self, client):
        if not self.deleteObjectFromCash(client):
            self.appendDict(aif.Client().addCmd(idclient=client.get('guid'), name=client.get('lastName')))
                # internal cards
            intcard = client.find('internal-card-type')
            for card in intcard.findall('internal-card'):
                    if not self.deleteObjectFromCash(card):
                        self.appendDict(aif.Card().addCmd(idcard=card.get('numberField'), idcardgroup=None, 
                                        number=card.get('numberField'), blocked=0, 
                                        discountpercent=card.get('discountpercent'), 
                                        cardsum=card.get('amount'), idclient=client.get('guid'), 
                                        validitydateend=card.get('expirationDate')))
                    else:
                        self.appendDict(aif.Card().deleteCmd(idcard=card.get('numberField')))
        else:
            self.appendDict(aif.Client().deleteCmd(idclient=client.get('guid')))

    def deleteVisualVerifyOrRequiredQuantity(self, productType):
        if productType == 'InputQuantEntity':
            self.appendDict({'command': 'clearTmcrequiredquantity'})
        elif productType == 'VisualAuthEntity':
            self.appendDict({'command': 'clearTmcvisualverify'})

    def parseVisualVerifyAndRequiredQuantity(self, goodCode, productType):
        if productType == 'InputQuantEntity':
            self.appendDict({'command': 'addTmcrequiredquantity', 'tmcrequiredquantity': {'tmccode' : goodCode}})
        elif productType == 'VisualAuthEntity':
            self.appendDict({'command': 'addTmcvisualverify', 'tmcvisualverify': {'tmccode' : goodCode}})

    def run(self):
        global errorObjectsCount
        result = self.inserter.start()

        if result:
            for dictFile in self.data:
                self.log.info(u"Парсинг файла %s" % dictFile)
                try:
                    root = ET.parse(dictFile).getroot()
                except Exception as e:
                    self.log.error(u'Файл %s имеет некорректный формат. %s' % (str(dictFile), str(e)))
                    errorList.append(u'Файл %s имеет некорректный формат. %s' % (str(dictFile), str(e)))
                try:
                    if root.tag == 'goods-catalog':
                        self.getMinPrices(root)
                        self.getMaxDiscounts(root)
                        
                        for good in root.findall('good'):
                            if self.simpleCond(good):
                                self.appendDict(self.parseGood(good))
                            #self.log.debug(self.listObjects)
                    elif root.tag == 'cards-catalog':
                        for client in root.findall('client'):
                            self.parseClientAndCards(client)
                    elif root.tag == 'goodsData':
                        productType = root.find('goods_type').find('product_type').text
#                        self.deleteVisualVerifyOrRequiredQuantity(productType)
                        for item in root.findall('goods'):
                            self.parseVisualVerifyAndRequiredQuantity(item.find('Code').text, productType)
                            #
                except Exception as e:
                    self.log.error(u'Ошибка в файле: %s. %s' % (str(dictFile), e.message))
                    errorList.append(u'Ошибка в файле: %s. %s' % (str(dictFile), e.message))


            # add prices
            for pr in self.prices:
                self.appendDict(pr)
            # add barcodes
            for bc in self.additionalBarcodes:
                self.appendDict(bc)
            # add adiitional prices
            for ap in self.additionalPrices:
                self.appendDict(ap)
            # add invent groups
            for gr in self.groups:
                self.appendDict(aif.InventGroup().addCmd(groupcode=gr, parentgroupcode=self.groups[gr][1], groupname=self.groups[gr][0]))

            if len(self.listObjects) > 0:
                    self.inserter.add_package(self.listObjects)
                    self.listObjects = []
        result = self.inserter.stop()
        errorObjectsCount = self.inserter.invalidObjectsCount()
        return result

def general_method(workDirectory, data, node):
    log = logger.getUploadLogger("Node", node.id)
    result = False
    global errorList
    errorList = []
    try:
        result = Converter(workDirectory, data, node).run()
    except InserterError as e:
        raise
    except Exception as e:
        errorList.append(e)
        log.exception(e)
    return result, errorList

def invalidObjectsCount():
    return errorObjectsCount
