# -*- coding: UTF-8 -*-

import os
from datetime import datetime
from common import macros, logger, rest_state
from qpid.messaging import *


def getCashPath(node, workDirectory):
    cashPath = node.currentConverter.cashPath
    if node.currentFlag and os.path.isfile(os.path.join(workDirectory, node.currentFlag)): 
        with open(os.path.join(workDirectory, node.currentFlag), 'r') as flag:
            path = os.path.normpath(flag.readline())
            if path and len(path) > 1:
                cashPath = path
    return cashPath


def general_method(workDirectory, data, node):
    errorList = []
    log = logger.getUploadLogger("Node", node.id)
    try:
        broker = '%s/%s@%s:%s' % (node.inserter.queue.username, node.inserter.queue.password, node.inserter.queue.host, str(node.inserter.queue.port))
        connection = Connection(broker, reconnect=True)
        connection.open()
        session = connection.session()
        sender = session.sender("topicCS.%s; {create: always, node: {type: topic, durable: True} }" % node.id, durable=True)
        for dataFile in data:
            cashPath = macros.applyMacros(str(getCashPath(node, workDirectory)), log, filename=os.path.basename(dataFile))
            log.info('Отправляем скрипт для удаления на кассах файла %s' % cashPath)
            sender.send(Message(content="if [ -f '%s' ];\nthen\nrm '%s'\nfi" % (cashPath, cashPath), id=hex(hash(datetime.now())), properties={"type": "EXECSCRIPT"}, durable=True))
            messageProperties = {"type": "DOWNLOAD", "fileLocation": "body", "localPath": cashPath}
            log.info('Отправляем файл %s на кассы' % cashPath)
            with open(dataFile, 'rb') as f:
                sender.send(Message(content=f.read(), id=hex(hash(datetime.now())), properties=messageProperties, durable=True))
        # можно послать сигнал на кассы, чтобы они забрали файлы, но пока не будем вызывать
        #for cashCode in node.currentConverter.cashCodes:
        #    rest_state.startLoadDictCash(cashCode, log)
        return True, errorList
    except Exception as e:
        log.error(u'Ошибка: %s' % str(e))
        errorList.append(u'Ошибка: %s' % str(e))
    return False, errorList


def invalidObjectsCount():
    return 0
