# -*- coding: UTF-8 -*-

from upload.inserter.queue import Inserter
from common import aifSoapClasses as soapCl, logger
from common.rest_state import RestState as rest_state
from collections import Iterable
import datetime
import traceback

class SoapConverter(object):

    def __init__(self, node):
        self.node = node
        self.objectsCount = 0
        self.inserter = None
        self.clearCmd = False
        self.clearIfEmptyDataCmd = False

    def convertToMarshalableType(self, value):
        if isinstance(value, datetime.datetime) or isinstance(value, datetime.date) or isinstance(value, datetime.time):
            value = str(value)
        return value

    def getCommand(self, clazz, obj, method, returnClass = False):
        dictParams = {}
        if callable(clazz):
            clazz = clazz()
        func = getattr(clazz, method)
        if callable(func):
            for paramName in func.func_code.co_varnames:
                if paramName != 'self' and paramName in dir(obj):
                    paramValue = getattr(obj, paramName)
                    dictParams[paramName] = self.converBaseAifToDict(paramValue)
            if returnClass:
                func(**dictParams)
                return clazz
            else:
                return func(**dictParams)

    def converBaseAifToDict(self, paramValue):
        result = None
        if isinstance(paramValue, soapCl.BaseAif):
            dictObj = {}
            for param in dir(paramValue):
                if param not in dir(soapCl.BaseAif):
                    value = getattr(paramValue, param)
                    if value != None:
                        dictObj[param] = self.converBaseAifToDict(value)
            result = dictObj
        elif isinstance(paramValue, Iterable) and not isinstance(paramValue, dict) and len(paramValue) > 0 and isinstance(paramValue[0], soapCl.BaseAif):
            result = [self.converBaseAifToDict(x) for x in paramValue]
        else:
            result = paramValue
        return self.convertToMarshalableType(result)

    def start(self):
        result = soapCl.DictConvertResult()
        result.result = False
    #    result.info = 'Ошибка на стадии блокировки БД'
        result.info = 'Error at the stage starting'
        self.inserter = Inserter(self.node)
        result.result = self.inserter.start()
        if not result.result:
            self.inserter.stop()
    #            result.info = 'Ошибка открытия транзакции для записи объектов справочника в БД'
                #result.info = 'Error start for writing dictionary objects to the DB'
        else:
            result.info = 'OK'
        return result

    def convert(self, objects, shopCode, aifCommandClass, curMethod, clear = False, clearIfEmptyData=False):
        log = logger.getUploadLogger("Node", self.node.id)
        hotKeyByBarcode = self.node.currentConverter.hotKeyByBarcode
        nonNoneObjects = objects if objects else []
        try:
            result = soapCl.DictConvertResult()
            result.result = False
        #    result.info = 'Ошибка на стадии начала конвертирования'
            result.info = 'Error at the stage start converting'
            listObjects = []
            self.clearCmd = clear
            self.clearIfEmptyDataCmd = clearIfEmptyData
            lenObjects = len(nonNoneObjects)
            if (clear and ((clearIfEmptyData and lenObjects == 0) or (not clearIfEmptyData and lenObjects > 0))):
                listObjects.append(self.getCommand(aifCommandClass, None, 'clearCmd'))
                self.objectsCount -= 1
            if (clearIfEmptyData and lenObjects > 0):
                result.info = 'You can use this option "clearIfEmptyData" only with an empty array of objects.'
                self.node.currentConverter.dict_state_id = rest_state.sendDictState(self.node, result.info, u'Вы можете использовать параметр "clearIfEmptyData" только с пустым массивом объектов.', rest_state.STATUS_ERROR, self.node.currentConverter.dict_state_id)
            else:
                for obj in nonNoneObjects:
                    cmd = self.getCommand(aifCommandClass, obj, curMethod)
                    if hotKeyByBarcode:
                        if 'addHotKey' == cmd['command'] and not 'bybarcode' in cmd['hotkey']:
                            cmd['hotkey']['bybarcode'] = hotKeyByBarcode
                    listObjects.append(cmd)
                    log.debug('Add command: %s' % cmd)
                    listObjectsSize = len(listObjects)
                    if listObjectsSize >= self.node.pack_size:
                        self.objectsCount += listObjectsSize
                        self.inserter.add_package(listObjects)
                        del(listObjects[:])
                listObjectsSize = len(listObjects)
                if listObjectsSize > 0:
                    self.objectsCount += listObjectsSize
                    self.inserter.add_package(listObjects)
                    del(listObjects[:])
                result.result = True
        except Exception, e:
    #         db.stop(self.node)
            messEn = u'Can not convert SOAP data to AIF or send dictionary to queue. Error: %s' % e.message
            messRu = u'Ошибка конвертирования или отправки справочника в очередь. Error: %s' % e.message
            log.error('%s %s' % (messEn, traceback.format_exc().decode('utf8')))
            self.node.currentConverter.dict_state_id = rest_state.sendDictState(self.node, messEn, messRu, rest_state.STATUS_ERROR, self.node.currentConverter.dict_state_id)
            result.result = False
            result.info = messEn
            return result
        if result.result:
            result.info = u'OK'
        else:
            self.node.currentConverter.dict_state_id = rest_state.sendDictState(self.node, u'Can not send dictionary to queue', u'Ошибка отправки справочника в очередь', rest_state.STATUS_ERROR, self.node.currentConverter.dict_state_id)
            if not result.info:
                result.info = 'Can not send dictionary to queue'
        return result

    def stop(self):
        result = soapCl.DictConvertResult()
        result.result = self.inserter.stop()
        if self.objectsCount == 0:
            if self.clearCmd:
                if self.clearIfEmptyDataCmd:
                    result.info = 'Sent dictionary clear command.'
                    self.node.currentConverter.dict_state_id = rest_state.sendDictState(self.node, result.info, u'Отправлена команда очистки справочника.', rest_state.STATUS_READY_WITH_ERROR, self.node.currentConverter.dict_state_id)
                else:
                    result.info = 'There is no confirmation of a complete cleaning dictionary(clear = true, clearIfEmptyData = true). Dictionary is not cleaned.'
                    self.node.currentConverter.dict_state_id = rest_state.sendDictState(self.node, result.info, u'Нет подтверждения полной очистки справочника(clear = true, clearIfEmptyData = true). Справочник не очищен.', rest_state.STATUS_ERROR, self.node.currentConverter.dict_state_id)
            else:
                result.info = 'No entries have been added to queue.'
                self.node.currentConverter.dict_state_id = rest_state.sendDictState(self.node, result.info, u'Ни одной записи не добавлено в очередь.', rest_state.STATUS_ERROR, self.node.currentConverter.dict_state_id)
            result.result = True
            return result
        result.info = u'OK. Objects count = %s' % self.objectsCount
        if result.result:
            self.node.currentConverter.dict_state_id = rest_state.sendDictState(self.node, u'OK. Objects count = %s' % self.objectsCount, u'ОК. Количество объектов = %s' % self.objectsCount, rest_state.STATUS_READY, self.node.currentConverter.dict_state_id)
    #       result.info = 'Ошибка закрытия транзакции записи объектов справочника в БД.'
        else:
            result.info = 'Error closing the transaction writting dictionary objects.'
            self.node.currentConverter.dict_state_id = rest_state.sendDictState(self.node, result.info, u'Ошибка записи справочника в очередь.', rest_state.STATUS_ERROR, self.node.currentConverter.dict_state_id)
        return result

    def rollback(self, sessionIdle=False):
        result = soapCl.DictConvertResult()
        result.result = self.inserter.rollback()
        result.info = u'OK'
        enPrefix = u''
        ruPrefix = u''
        if sessionIdle:
            enPrefix = u'Exceeded session idle timeout. '
            ruPrefix = u'Превышено время ожидания сессии. '
        if result.result:
            self.node.currentConverter.dict_state_id = rest_state.sendDictState(self.node, u'%sCancel dictionary loading' % enPrefix, u'%sЗагрузка справочника отменена' % ruPrefix, rest_state.STATUS_ERROR, self.node.currentConverter.dict_state_id)
        else:
            self.node.currentConverter.dict_state_id = rest_state.sendDictState(self.node, u'%sError canceling dictionary loading' % enPrefix, u'%sОшибка отмены загрузки справочника' % ruPrefix, rest_state.STATUS_ERROR, self.node.currentConverter.dict_state_id)
            result.info = 'Error rollback the transaction writting dictionary objects.'
        return result
